@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,16 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2020
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - Martin Barisits <martin.barisits@cern.ch>, 2020-2021
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020
-# - martynia <janusz.martyniak@googlemail.com>, 2021
-# - Janusz Martyniak <janusz.martyniak@googlemail.com>, 2022
-
-from __future__ import print_function
 
 from rucio.api.permission import has_permission
 from rucio.api.scope import list_scopes
@@ -30,9 +20,11 @@ from rucio.core.rse import get_rse_id
 from rucio.core import dirac
 from rucio.common.exception import AccessDenied
 from rucio.common.utils import extract_scope
+from rucio.db.sqla.session import transactional_session
 
 
-def add_files(lfns, issuer, ignore_availability, vo='def'):
+@transactional_session
+def add_files(lfns, issuer, ignore_availability, vo='def', session=None):
     """
     Bulk add files :
     - Create the file and replica.
@@ -43,9 +35,10 @@ def add_files(lfns, issuer, ignore_availability, vo='def'):
     :param issuer: The issuer account.
     :param ignore_availability: A boolean to ignore blocked sites.
     :param vo: The VO to act on.
+    :param session: The database session in use.
 
     """
-    scopes = list_scopes(vo=vo)
+    scopes = list_scopes(vo=vo, session=session)
     dids = []
     rses = {}
     for lfn in lfns:
@@ -53,7 +46,7 @@ def add_files(lfns, issuer, ignore_availability, vo='def'):
         dids.append({'scope': scope, 'name': name})
         rse = lfn['rse']
         if rse not in rses:
-            rse_id = get_rse_id(rse=rse, vo=vo)
+            rse_id = get_rse_id(rse=rse, vo=vo, session=session)
             rses[rse] = rse_id
         lfn['rse_id'] = rses[rse]
 
@@ -61,14 +54,14 @@ def add_files(lfns, issuer, ignore_availability, vo='def'):
     for rse in rses:
         rse_id = rses[rse]
         kwargs = {'rse': rse, 'rse_id': rse_id}
-        if not has_permission(issuer=issuer, action='add_replicas', kwargs=kwargs, vo=vo):
+        if not has_permission(issuer=issuer, action='add_replicas', kwargs=kwargs, vo=vo, session=session):
             raise AccessDenied('Account %s can not add file replicas on %s for VO %s' % (issuer, rse, vo))
-        if not has_permission(issuer=issuer, action='skip_availability_check', kwargs=kwargs, vo=vo):
+        if not has_permission(issuer=issuer, action='skip_availability_check', kwargs=kwargs, vo=vo, session=session):
             ignore_availability = False
 
     # Check if the issuer can add the files
     kwargs = {'issuer': issuer, 'dids': dids}
-    if not has_permission(issuer=issuer, action='add_dids', kwargs=kwargs, vo=vo):
+    if not has_permission(issuer=issuer, action='add_dids', kwargs=kwargs, vo=vo, session=session):
         raise AccessDenied('Account %s can not bulk add data identifier for VO %s' % (issuer, vo))
 
-    dirac.add_files(lfns=lfns, account=issuer, ignore_availability=ignore_availability, session=None, vo=vo)
+    dirac.add_files(lfns=lfns, account=issuer, ignore_availability=ignore_availability, vo=vo, session=session)
