@@ -1,4 +1,5 @@
-# Copyright 2015-2019 CERN for the benefit of the ATLAS collaboration.
+# -*- coding: utf-8 -*-
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,15 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Fernando Lopez <fernando.e.lopez@gmail.com>, 2015
-# - Joaquin Bogado <jbogado@linti.unlp.edu.ar>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018-2019
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2019
-# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2020
-#
-# PY3K COMPATIBLE
 
 import datetime
 import logging
@@ -56,11 +48,7 @@ class Consistency(data_models.DataModel):
         else:
             assert subcommand == 'consistency-manual'
 
-        prefix = path_parsing.prefix(
-            dumper.agis_endpoints_data(),
-            ddm_endpoint,
-        )
-        prefix_components = path_parsing.components(prefix)
+        prefix_components = path_parsing.components(dumper.ddmendpoint_url(ddm_endpoint))
 
         def parser(line):
             '''
