@@ -18,21 +18,20 @@ import json
 import logging
 import traceback
 from collections import namedtuple
-from configparser import NoOptionError, NoSectionError
 from typing import TYPE_CHECKING
 
-from sqlalchemy import and_, or_, func, update, select, delete
+from sqlalchemy import and_, or_, update, select, delete, exists
 from sqlalchemy.exc import IntegrityError
 from sqlalchemy.orm import aliased
-from sqlalchemy.sql.expression import asc, true, false, null
+from sqlalchemy.sql.expression import asc, true, false, null, func
 
-from rucio.common.config import config_get_bool, config_get
+from rucio.common.config import config_get_bool
 from rucio.common.exception import RequestNotFound, RucioException, UnsupportedOperation
 from rucio.common.types import InternalAccount, InternalScope
-from rucio.common.utils import generate_uuid, chunks, get_parsed_throttler_mode
+from rucio.common.utils import generate_uuid, chunks
 from rucio.core.message import add_message, add_messages
 from rucio.core.monitor import record_counter, record_timer
-from rucio.core.rse import get_rse_name, get_rse_vo, get_rse_transfer_limits, get_rse_attribute, RseData
+from rucio.core.rse import get_rse_name, get_rse_vo, RseData
 from rucio.db.sqla import models, filter_thread_work
 from rucio.db.sqla.constants import RequestState, RequestType, LockState, RequestErrMsg, ReplicaState
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
@@ -41,13 +40,9 @@ from rucio.db.sqla.util import temp_table_mngr
 RequestAndState = namedtuple('RequestAndState', ['request_id', 'request_state'])
 
 if TYPE_CHECKING:
-    from typing import Any, Dict, Iterator, List, Optional, Callable, Set, Union, Sequence
-    from sqlalchemy.orm import Session
+    from typing import Any, Dict, List, Optional, Sequence, Union
 
-    RequestResult = Dict[str, Any]
-    RequestResultOrState = Union[RequestResult, RequestAndState]
-    RowIterator = Iterator[RequestResult]
-    LoggerFunction = Callable[..., Any]
+    from sqlalchemy.orm import Session
 
 """
 The core request.py is specifically for handling requests.
@@ -69,8 +64,26 @@ class RequestSource:
 
 
 class RequestWithSources:
-    def __init__(self, id_, request_type, rule_id, scope, name, md5, adler32, byte_count, activity, attributes,
-                 previous_attempt_id, dest_rse_data, account, retry_count, priority, transfertool, requested_at=None):
+    def __init__(
+            self,
+            id_: "Optional[str]",
+            request_type: RequestType,
+            rule_id: "Optional[str]",
+            scope: InternalScope,
+            name: str,
+            md5: str,
+            adler32: str,
+            byte_count: int,
+            activity: str,
+            attributes: "Union[str, None, Dict[str, Any]]",
+            previous_attempt_id: "Optional[str]",
+            dest_rse_data: RseData,
+            account: InternalAccount,
+            retry_count: int,
+            priority: int,
+            transfertool: str,
+            requested_at: "Optional[datetime.datetime]" = None,
+    ):
 
         self.request_id = id_
         self.request_type = request_type
@@ -91,7 +104,8 @@ class RequestWithSources:
         self.transfertool = transfertool
         self.requested_at = requested_at if requested_at else datetime.datetime.utcnow()
 
-        self.sources = []
+        self.sources: "List[RequestSource]" = []
+        self.requested_source: "Optional[RequestSource]" = None
 
     def __str__(self):
         return "{}({}:{})".format(self.request_id, self.scope, self.name)
@@ -328,36 +342,36 @@ def queue_requests(requests, session=None, logger=logging.log):
 
 @read_session
 def list_transfer_requests_and_source_replicas(
-        total_workers=0,
-        worker_number=0,
-        partition_hash_var=None,
-        limit=None,
-        activity=None,
-        older_than=None,
-        rses=None,
-        multihop_rses=None,
-        request_type=RequestType.TRANSFER,
-        request_state=None,
-        ignore_availability=False,
-        transfertool=None,
-        session=None,
-) -> "List[RequestWithSources]":
+        total_workers: int = 0,
+        worker_number: int = 0,
+        partition_hash_var: "Optional[str]" = None,
+        limit: "Optional[int]" = None,
+        activity: "Optional[str]" = None,
+        older_than: "Optional[datetime.datetime]" = None,
+        rses: "Optional[Sequence[str]]" = None,
+        request_type: "Optional[List[RequestType]]" = None,
+        request_state: "Optional[RequestState]" = None,
+        required_source_rse_attrs: "Optional[List[str]]" = None,
+        ignore_availability: bool = False,
+        transfertool: "Optional[str]" = None,
+        session: "Optional[Session]" = None,
+) -> "Dict[str, RequestWithSources]":
     """
     List requests with source replicas
-    :param total_workers:      Number of total workers.
-    :param worker_number:      Id of the executing worker.
-    :param partition_hash_var  The hash variable used for partitioning thread work
-    :param limit:              Integer of requests to retrieve.
-    :param activity:           Activity to be selected.
-    :param older_than:         Only select requests older than this DateTime.
-    :param rses:               List of rse_id to select requests.
-    :param multihop_rses:               List of rse_id allowed to be used for multihop
-    :param request_type:       Filter on the given request type.
-    :param request_state:      Filter on the given request state
-    :param transfertool:       The transfer tool as specified in rucio.cfg.
-    :param ignore_availability Ignore blocklisted RSEs
-    :param session:            Database session to use.
-    :returns:                  List of RequestWithSources objects.
+    :param total_workers: Number of total workers.
+    :param worker_number: Id of the executing worker.
+    :param partition_hash_var: The hash variable used for partitioning thread work
+    :param limit: Integer of requests to retrieve.
+    :param activity: Activity to be selected.
+    :param older_than: Only select requests older than this DateTime.
+    :param rses: List of rse_id to select requests.
+    :param request_type: Filter on the given request type.
+    :param request_state: Filter on the given request state
+    :param transfertool: The transfer tool as specified in rucio.cfg.
+    :param required_source_rse_attrs: Only select source RSEs having these attributes set
+    :param ignore_availability: Ignore blocklisted RSEs
+    :param session: Database session to use.
+    :returns: List of RequestWithSources objects.
     """
 
     if partition_hash_var is None:
@@ -366,8 +380,12 @@ def list_transfer_requests_and_source_replicas(
     if request_state is None:
         request_state = RequestState.QUEUED
 
+    if request_type is None:
+        request_type = [RequestType.TRANSFER]
+
     sub_requests = select(
         models.Request.id,
+        models.Request.request_type,
         models.Request.rule_id,
         models.Request.scope,
         models.Request.name,
@@ -377,6 +395,7 @@ def list_transfer_requests_and_source_replicas(
         models.Request.activity,
         models.Request.attributes,
         models.Request.previous_attempt_id,
+        models.Request.source_rse_id,
         models.Request.dest_rse_id,
         models.Request.retry_count,
         models.Request.account,
@@ -388,7 +407,7 @@ def list_transfer_requests_and_source_replicas(
         models.Request, "INDEX(REQUESTS REQUESTS_TYP_STA_UPD_IDX)", 'oracle'
     ).where(
         models.Request.state == request_state,
-        models.Request.request_type == request_type
+        models.Request.request_type.in_(request_type)
     ).join(
         models.RSE,
         models.RSE.id == models.Request.dest_rse_id
@@ -436,6 +455,7 @@ def list_transfer_requests_and_source_replicas(
 
     stmt = select(
         sub_requests.c.id,
+        sub_requests.c.request_type,
         sub_requests.c.rule_id,
         sub_requests.c.scope,
         sub_requests.c.name,
@@ -445,14 +465,15 @@ def list_transfer_requests_and_source_replicas(
         sub_requests.c.activity,
         sub_requests.c.attributes,
         sub_requests.c.previous_attempt_id,
+        sub_requests.c.source_rse_id,
         sub_requests.c.dest_rse_id,
         sub_requests.c.account,
         sub_requests.c.retry_count,
         sub_requests.c.priority,
         sub_requests.c.transfertool,
         sub_requests.c.requested_at,
-        models.RSE.id.label("source_rse_id"),
-        models.RSE.rse,
+        models.RSE.id.label("replica_rse_id"),
+        models.RSE.rse.label("replica_rse_name"),
         models.RSEFileAssociation.path,
         models.Source.ranking.label("source_ranking"),
         models.Source.url.label("source_url"),
@@ -485,18 +506,22 @@ def list_transfer_requests_and_source_replicas(
         models.Distance, "INDEX(DISTANCES DISTANCES_PK)", 'oracle'
     )
 
-    # if transfertool specified, select only the requests where the source rses are set up for the transfer tool
-    if transfertool and not multihop_rses:
-        stmt = stmt.join(
-            models.RSEAttrAssociation, models.RSEAttrAssociation.rse_id == models.RSE.id,  # source_rse_id
-        ).where(
-            models.RSEAttrAssociation.key == 'transfertool',
-            models.RSEAttrAssociation.value.like('%' + transfertool + '%')
+    for attribute in required_source_rse_attrs or ():
+        rse_attr_alias = aliased(models.RSEAttrAssociation)
+        stmt = stmt.where(
+            exists(
+                select(
+                    [1]
+                ).where(
+                    rse_attr_alias.rse_id == models.RSE.id,
+                    rse_attr_alias.key == attribute
+                )
+            )
         )
 
     requests_by_id = {}
-    for (request_id, rule_id, scope, name, md5, adler32, byte_count, activity, attributes, previous_attempt_id, dest_rse_id, account, retry_count,
-         priority, transfertool, requested_at, source_rse_id, source_rse_name, file_path, source_ranking, source_url, distance_ranking) in session.execute(stmt):
+    for (request_id, req_type, rule_id, scope, name, md5, adler32, byte_count, activity, attributes, previous_attempt_id, source_rse_id, dest_rse_id, account, retry_count,
+         priority, transfertool, requested_at, replica_rse_id, replica_rse_name, file_path, source_ranking, source_url, distance_ranking) in session.execute(stmt):
 
         # If we didn't pre-filter using temporary tables on database side, perform the filtering here
         if not use_temp_tables and rses and dest_rse_id not in rses:
@@ -504,17 +529,20 @@ def list_transfer_requests_and_source_replicas(
 
         request = requests_by_id.get(request_id)
         if not request:
-            request = RequestWithSources(id_=request_id, request_type=request_type, rule_id=rule_id, scope=scope, name=name,
+            request = RequestWithSources(id_=request_id, request_type=req_type, rule_id=rule_id, scope=scope, name=name,
                                          md5=md5, adler32=adler32, byte_count=byte_count, activity=activity, attributes=attributes,
                                          previous_attempt_id=previous_attempt_id, dest_rse_data=RseData(id_=dest_rse_id),
                                          account=account, retry_count=retry_count, priority=priority, transfertool=transfertool,
                                          requested_at=requested_at)
             requests_by_id[request_id] = request
 
-        if source_rse_id is not None:
-            request.sources.append(RequestSource(rse_data=RseData(id_=source_rse_id, name=source_rse_name), file_path=file_path,
-                                                 source_ranking=source_ranking, distance_ranking=distance_ranking, url=source_url))
-    return list(requests_by_id.values())
+        if replica_rse_id is not None:
+            source = RequestSource(rse_data=RseData(id_=replica_rse_id, name=replica_rse_name), file_path=file_path,
+                                   source_ranking=source_ranking, distance_ranking=distance_ranking, url=source_url)
+            request.sources.append(source)
+            if source_rse_id == replica_rse_id:
+                request.requested_source = source
+    return requests_by_id
 
 
 @read_session
@@ -1166,13 +1194,11 @@ def get_heavy_load_rses(threshold, session=None):
 
 
 @read_session
-def get_stats_by_activity_direction_state(state, all_activities=False, direction='destination', session=None):
+def get_request_stats(state, session=None):
     """
     Retrieve statistics about requests by destination, activity and state.
 
     :param state:           Request state.
-    :param all_activities:  Boolean whether requests are grouped by activity or if activities are ignored.
-    :param direction:       Direction if requests are grouped by source RSE or destination RSE.
     :param session:         Database session to use.
     :returns:               List of (activity, dest_rse_id, state, counter).
     """
@@ -1181,47 +1207,25 @@ def get_stats_by_activity_direction_state(state, all_activities=False, direction
         state = [state]
 
     try:
-        rse_id_column = models.Request.source_rse_id if direction == 'source' else models.Request.dest_rse_id
-
-        additional_columns = []
-        if not all_activities:
-            additional_columns = [
-                models.Request.activity
-            ]
-
-        subquery = select(
+        stmt = select(
             models.Request.account,
             models.Request.state,
+            models.Request.dest_rse_id,
+            models.Request.source_rse_id,
+            models.Request.activity,
             func.count(1).label('counter'),
-            rse_id_column.label('rse_id'),
-            *additional_columns,
         ).with_hint(
             models.Request, "INDEX(REQUESTS REQUESTS_TYP_STA_UPD_IDX)", 'oracle'
         ).where(
-            models.Request.state.in_(state)
+            models.Request.state.in_(state),
+            models.Request.request_type.in_([RequestType.TRANSFER, RequestType.STAGEIN, RequestType.STAGEOUT])
         ).group_by(
             models.Request.account,
             models.Request.state,
-            rse_id_column,
-            *additional_columns,
-        ).subquery()
-
-        stmt = select(
-            subquery.c.account,
-            subquery.c.state,
-            models.RSE.rse,
-            subquery.c.counter,
-            subquery.c.rse_id
-        ).with_hint(
-            models.RSE, "INDEX(RSES RSES_PK)", 'oracle'
-        ).join(
-            models.RSE,
-            models.RSE.id == subquery.c.rse_id,
+            models.Request.dest_rse_id,
+            models.Request.source_rse_id,
+            models.Request.activity,
         )
-        if not all_activities:
-            stmt = stmt.add_columns(
-                subquery.c.activity
-            )
 
         return session.execute(stmt).all()
 
@@ -1230,17 +1234,23 @@ def get_stats_by_activity_direction_state(state, all_activities=False, direction
 
 
 @transactional_session
-def release_waiting_requests_per_deadline(rse_id=None, deadline=1, session=None):
+def release_waiting_requests_per_deadline(
+        dest_rse_id: "Optional[str]" = None,
+        source_rse_id: "Optional[str]" = None,
+        deadline: int = 1,
+        session=None,
+):
     """
     Release waiting requests that were waiting too long and exceeded the maximum waiting time to be released.
     If the DID of a request is attached to a dataset, the oldest requested_at date of all requests related to the dataset will be used for checking and all requests of this dataset will be released.
-    :param rse_id:           The source RSE id.
-    :param deadline:         Maximal waiting time in hours until a dataset gets released.
-    :param session:          The database session.
+    :param dest_rse_id: The destination RSE id.
+    :param source_rse_id: The source RSE id.
+    :param deadline: Maximal waiting time in hours until a dataset gets released.
+    :param session: The database session.
     """
     amount_released_requests = 0
     if deadline:
-        grouped_requests_subquery, filtered_requests_subquery = create_base_query_grouped_fifo(rse_id, filter_by_rse='source', session=session)
+        grouped_requests_subquery, filtered_requests_subquery = create_base_query_grouped_fifo(dest_rse_id=dest_rse_id, source_rse_id=source_rse_id, session=session)
         old_requests_subquery = select(
             grouped_requests_subquery.c.name,
             grouped_requests_subquery.c.scope,
@@ -1270,12 +1280,18 @@ def release_waiting_requests_per_deadline(rse_id=None, deadline=1, session=None)
 
 
 @transactional_session
-def release_waiting_requests_per_free_volume(rse_id, volume=None, session=None):
+def release_waiting_requests_per_free_volume(
+        dest_rse_id: "Optional[str]" = None,
+        source_rse_id: "Optional[str]" = None,
+        volume: int = 0,
+        session=None
+):
     """
     Release waiting requests if they fit in available transfer volume. If the DID of a request is attached to a dataset, the volume will be checked for the whole dataset as all requests related to this dataset will be released.
 
-    :param rse_id:  The destination RSE id.
-    :param volume:  The maximum volume in bytes that should be transfered.
+    :param dest_rse_id: The destination RSE id.
+    :param source_rse_id: The source RSE id
+    :param volume: The maximum volume in bytes that should be transfered.
     :param session: The database session.
     """
 
@@ -1290,13 +1306,19 @@ def release_waiting_requests_per_free_volume(rse_id, volume=None, session=None):
     sum_volume_active_subquery = select(
         coalesce_func(func.sum(models.Request.bytes), 0).label('sum_bytes')
     ).where(
-        and_(
-            or_(models.Request.state == RequestState.SUBMITTED, models.Request.state == RequestState.QUEUED),
-            models.Request.dest_rse_id == rse_id
+        models.Request.state.in_([RequestState.SUBMITTED, RequestState.QUEUED]),
+    )
+    if dest_rse_id is not None:
+        sum_volume_active_subquery = sum_volume_active_subquery.where(
+            models.Request.dest_rse_id == dest_rse_id
         )
-    ).subquery()
+    if source_rse_id is not None:
+        sum_volume_active_subquery = sum_volume_active_subquery.where(
+            models.Request.source_rse_id == source_rse_id
+        )
+    sum_volume_active_subquery = sum_volume_active_subquery.subquery()
 
-    grouped_requests_subquery, filtered_requests_subquery = create_base_query_grouped_fifo(rse_id, filter_by_rse='destination', session=session)
+    grouped_requests_subquery, filtered_requests_subquery = create_base_query_grouped_fifo(dest_rse_id=dest_rse_id, source_rse_id=source_rse_id, session=session)
 
     cumulated_volume_subquery = select(
         grouped_requests_subquery.c.name,
@@ -1329,14 +1351,18 @@ def release_waiting_requests_per_free_volume(rse_id, volume=None, session=None):
 
 
 @read_session
-def create_base_query_grouped_fifo(rse_id, filter_by_rse='destination', session=None):
+def create_base_query_grouped_fifo(
+        dest_rse_id: "Optional[str]" = None,
+        source_rse_id: "Optional[str]" = None,
+        session=None
+):
     """
     Build the sqlalchemy queries to filter relevant requests and to group them in datasets.
     Group requests either by same destination RSE or source RSE.
 
-    :param rse_id:           The RSE id.
-    :param filter_by_rse:    Decide whether to filter by transfer destination or source RSE (`destination`, `source`).
-    :param session:          The database session.
+    :param dest_rse_id: The source RSE id to filter on
+    :param source_rse_id: The destination RSE id to filter on
+    :param session: The database session.
     """
     dialect = session.bind.dialect.name
     if dialect == 'mysql' or dialect == 'sqlite':
@@ -1370,14 +1396,13 @@ def create_base_query_grouped_fifo(rse_id, filter_by_rse='destination', session=
     ).where(
         models.Request.state == RequestState.WAITING,
     )
-    # depending if throttler is used for reading or writing
-    if filter_by_rse == 'source':
+    if source_rse_id is not None:
         requests_subquery_stmt = requests_subquery_stmt.where(
-            models.Request.source_rse_id == rse_id
+            models.Request.source_rse_id == source_rse_id
         )
-    elif filter_by_rse == 'destination':
+    if dest_rse_id is not None:
         requests_subquery_stmt = requests_subquery_stmt.where(
-            models.Request.dest_rse_id == rse_id
+            models.Request.dest_rse_id == dest_rse_id
         )
 
     filtered_requests_subquery = requests_subquery_stmt.add_columns(
@@ -1408,16 +1433,23 @@ def create_base_query_grouped_fifo(rse_id, filter_by_rse='destination', session=
 
 
 @transactional_session
-def release_waiting_requests_fifo(rse_id, activity=None, count=None, account=None, direction='destination', session=None):
+def release_waiting_requests_fifo(
+        dest_rse_id: "Optional[str]" = None,
+        source_rse_id: "Optional[str]" = None,
+        activity: "Optional[str]" = None,
+        count: int = 0,
+        account: "Optional[InternalAccount]" = None,
+        session=None
+):
     """
     Release waiting requests. Transfer requests that were requested first, get released first (FIFO).
 
-    :param rse_id:           The RSE id.
-    :param activity:         The activity.
-    :param count:            The count to be released.
-    :param account:          The account name whose requests to release.
-    :param direction:        Direction if requests are grouped by source RSE or destination RSE.
-    :param session:          The database session.
+    :param source_rse_id: The source rse id
+    :param dest_rse_id: The destination rse id
+    :param activity: The activity.
+    :param count: The count to be released.
+    :param account: The account name whose requests to release.
+    :param session: The database session.
     """
 
     dialect = session.bind.dialect.name
@@ -1432,14 +1464,14 @@ def release_waiting_requests_fifo(rse_id, activity=None, count=None, account=Non
     ).limit(
         count
     )
-    if direction == 'destination':
-        subquery = subquery.where(models.Request.dest_rse_id == rse_id)
-    elif direction == 'source':
-        subquery = subquery.where(models.Request.source_rse_id == rse_id)
+    if source_rse_id is not None:
+        subquery = subquery.where(models.Request.source_rse_id == source_rse_id)
+    if dest_rse_id is not None:
+        subquery = subquery.where(models.Request.dest_rse_id == dest_rse_id)
 
-    if activity:
+    if activity is not None:
         subquery = subquery.where(models.Request.activity == activity)
-    if account:
+    if account is not None:
         subquery = subquery.where(models.Request.account == account)
 
     subquery = subquery.subquery()
@@ -1471,27 +1503,34 @@ def release_waiting_requests_fifo(rse_id, activity=None, count=None, account=Non
 
 
 @transactional_session
-def release_waiting_requests_grouped_fifo(rse_id, count=None, direction='destination', deadline=1, volume=0, session=None):
+def release_waiting_requests_grouped_fifo(
+        dest_rse_id: "Optional[str]" = None,
+        source_rse_id: "Optional[str]" = None,
+        count: int = 0,
+        deadline: int = 1,
+        volume: int = 0,
+        session=None
+):
     """
     Release waiting requests. Transfer requests that were requested first, get released first (FIFO).
     Also all requests to DIDs that are attached to the same dataset get released, if one children of the dataset is choosed to be released (Grouped FIFO).
 
-    :param rse_id:           The RSE id.
-    :param count:            The count to be released. If None, release all waiting requests.
-    :param direction:        Direction if requests are grouped by source RSE or destination RSE.
-    :param deadline:         Maximal waiting time in hours until a dataset gets released.
-    :param volume:           The maximum volume in bytes that should be transfered.
-    :param session:          The database session.
+    :param dest_rse_id: The destination rse id
+    :param source_rse_id: The source RSE id.
+    :param count: The count to be released. If None, release all waiting requests.
+    :param deadline: Maximal waiting time in hours until a dataset gets released.
+    :param volume: The maximum volume in bytes that should be transfered.
+    :param session: The database session.
     """
 
     amount_updated_requests = 0
 
     # Release requests that exceeded waiting time
-    if deadline:
-        amount_updated_requests = release_waiting_requests_per_deadline(rse_id=rse_id, deadline=deadline, session=session)
+    if deadline and source_rse_id is not None:
+        amount_updated_requests = release_waiting_requests_per_deadline(dest_rse_id=dest_rse_id, source_rse_id=source_rse_id, deadline=deadline, session=session)
         count = count - amount_updated_requests
 
-    grouped_requests_subquery, filtered_requests_subquery = create_base_query_grouped_fifo(rse_id=rse_id, filter_by_rse=direction, session=session)
+    grouped_requests_subquery, filtered_requests_subquery = create_base_query_grouped_fifo(dest_rse_id=dest_rse_id, source_rse_id=source_rse_id, session=session)
 
     # cumulate amount of children per dataset and combine with each request and only keep requests that dont exceed the limit
     cumulated_children_subquery = select(
@@ -1526,40 +1565,52 @@ def release_waiting_requests_grouped_fifo(rse_id, count=None, direction='destina
     amount_updated_requests += session.execute(stmt).rowcount
 
     # release requests where the whole datasets volume fits in the available volume space
-    if volume:
-        amount_updated_requests += release_waiting_requests_per_free_volume(rse_id=rse_id, volume=volume, session=session)
+    if volume and dest_rse_id is not None:
+        amount_updated_requests += release_waiting_requests_per_free_volume(dest_rse_id=dest_rse_id, volume=volume, session=session)
 
     return amount_updated_requests
 
 
 @transactional_session
-def release_all_waiting_requests(rse_id, activity=None, account=None, direction='destination', session=None):
+def release_all_waiting_requests(
+        dest_rse_id: "Optional[str]" = None,
+        source_rse_id: "Optional[str]" = None,
+        activity: "Optional[str]" = None,
+        account: "Optional[InternalAccount]" = None,
+        session=None
+):
     """
     Release all waiting requests per destination RSE.
 
-    :param rse_id:           The RSE id.
-    :param activity:         The activity.
-    :param account:          The account name whose requests to release.
-    :param direction:        Direction if requests are grouped by source RSE or destination RSE.
-    :param session:          The database session.
+    :param dest_rse_id: The destination rse id.
+    :param source_rse_id: The source rse id.
+    :param activity: The activity.
+    :param account: The account name whose requests to release.
+    :param session: The database session.
     """
     try:
-        rse_id_column = models.Request.source_rse_id if direction == 'source' else models.Request.dest_rse_id
         query = update(
             models.Request
         ).where(
             models.Request.state == RequestState.WAITING,
-            rse_id_column == rse_id
         ).execution_options(
             synchronize_session=False
         ).values(
             {'state': RequestState.QUEUED}
         )
-        if activity:
+        if source_rse_id is not None:
+            query = query.where(
+                models.Request.source_rse_id == source_rse_id
+            )
+        if dest_rse_id is not None:
+            query = query.where(
+                models.Request.dest_rse_id == dest_rse_id
+            )
+        if activity is not None:
             query = query.where(
                 models.Request.activity == activity
             )
-        if account:
+        if account is not None:
             query = query.where(
                 models.Request.account == account
             )
@@ -1901,119 +1952,3 @@ def list_requests_history(src_rse_ids, dst_rse_ids, states=None, offset=None, li
         stmt = stmt.limit(limit)
     for request in session.execute(stmt).yield_per(500).scalars():
         yield request
-
-
-@transactional_session
-def prepare_requests(
-        requests_with_sources: "Sequence[RequestWithSources]",
-        logger: "LoggerFunction" = logging.log,
-        session: "Optional[Session]" = None,
-) -> int:
-    """
-    Update transfer requests according to preparer settings.
-    """
-    count = 0
-    reqs_no_source = set()
-    for rws in requests_with_sources:
-        # Assume request doesn't have any sources. Will be removed later if sources are found.
-        reqs_no_source.add(rws.request_id)
-
-        if not rws.sources:
-            logger(logging.INFO, '%s: has no sources. Skipping.', rws)
-            continue
-
-        sources = sorted(rws.sources, key=lambda s: s.distance_ranking)
-
-        selected_source = None
-        transfertool = None
-        dest_rse_transfertools = get_supported_transfertools(rws.dest_rse.id, session=session)
-        for source in sources:
-            src_rse_transfertools = get_supported_transfertools(source.rse.id, session=session)
-            common_transfertools = dest_rse_transfertools.intersection(src_rse_transfertools)
-            if not common_transfertools:
-                continue
-            transfertool = 'fts3' if 'fts3' in common_transfertools else common_transfertools.pop()
-            selected_source = source
-
-        if not selected_source:
-            logger(logging.WARNING, '%s: all available sources were filtered', rws)
-            continue
-
-        reqs_no_source.remove(rws.request_id)
-
-        update_dict = {
-            models.Request.state: __throttler_request_state(
-                activity=rws.activity,
-                source_rse_id=selected_source.rse.id,
-                dest_rse_id=rws.dest_rse.id,
-                session=session,
-            ),
-            models.Request.source_rse_id: selected_source.rse.id,
-        }
-        if transfertool:
-            update_dict[models.Request.transfertool] = transfertool
-
-        stmt = update(
-            models.Request
-        ).where(
-            models.Request.id == rws.request_id
-        ).execution_options(
-            synchronize_session=False
-        ).values(
-            update_dict
-        )
-        session.execute(stmt)
-        count += 1
-
-    if reqs_no_source:
-        logger(logging.INFO, "Marking requests as no-sources: %s", reqs_no_source)
-        set_requests_state_if_possible(reqs_no_source, RequestState.NO_SOURCES, logger=logger, session=session)
-    return count
-
-
-def __throttler_request_state(activity, source_rse_id, dest_rse_id, session: "Optional[Session]" = None) -> RequestState:
-    """
-    Takes request attributes to return a new state for the request
-    based on throttler settings. Always returns QUEUED,
-    if the throttler mode is not set.
-    """
-    try:
-        throttler_mode = config_get('throttler', 'mode', default=None, use_cache=False, session=session)
-    except (NoOptionError, NoSectionError, RuntimeError):
-        throttler_mode = None
-
-    limit_found = False
-    if throttler_mode:
-        transfer_limits = get_rse_transfer_limits(session=session)
-        activity_limit = transfer_limits.get(activity, {})
-        all_activities_limit = transfer_limits.get('all_activities', {})
-        direction, all_activities = get_parsed_throttler_mode(throttler_mode)
-        if direction == 'source':
-            if all_activities:
-                if all_activities_limit.get(source_rse_id):
-                    limit_found = True
-            else:
-                if activity_limit.get(source_rse_id):
-                    limit_found = True
-        elif direction == 'destination':
-            if all_activities:
-                if all_activities_limit.get(dest_rse_id):
-                    limit_found = True
-            else:
-                if activity_limit.get(dest_rse_id):
-                    limit_found = True
-
-    return RequestState.WAITING if limit_found else RequestState.QUEUED
-
-
-def get_supported_transfertools(rse_id: str, session=None) -> "Set[str]":
-    transfertool_attr = get_rse_attribute(rse_id, 'transfertool', session=session)
-
-    if not transfertool_attr:
-        return {'fts3', 'globus'}
-
-    result = set()
-    for transfertool in transfertool_attr.split(sep=','):
-        if transfertool.strip():
-            result.add(transfertool)
-    return result
