@@ -16,6 +16,7 @@
 import datetime
 import logging
 import re
+from typing import TYPE_CHECKING
 from configparser import NoOptionError, NoSectionError
 
 from json import dumps
@@ -31,9 +32,25 @@ from rucio.db.sqla import models
 from rucio.db.sqla.constants import SubscriptionState
 from rucio.db.sqla.session import transactional_session, stream_session, read_session
 
+if TYPE_CHECKING:
+    from typing import Any, Dict, Iterator, Optional, Callable
+    from sqlalchemy.orm import Session
+    from rucio.common.types import InternalAccount
+    LoggerFunction = Callable[..., Any]
+    SubscriptionType = Dict
+
 
 @transactional_session
-def add_subscription(name, account, filter_, replication_rules, comments, lifetime, retroactive, dry_run, priority=3, session=None):
+def add_subscription(name: str,
+                     account: "InternalAccount",
+                     filter_: str,
+                     replication_rules: str,
+                     comments: str,
+                     lifetime: "Optional[int]" = None,
+                     retroactive: "Optional[bool]" = False,
+                     dry_run: "Optional[bool]" = False,
+                     priority: "Optional[int]" = 3,
+                     session: "Optional[Session]" = None) -> str:
     """
     Adds a new subscription which will be verified against every new added file and dataset
 
@@ -68,17 +85,18 @@ def add_subscription(name, account, filter_, replication_rules, comments, lifeti
     SubscriptionHistory = models.SubscriptionHistory
     retroactive = bool(retroactive)  # Force boolean type, necessary for strict SQL
     state = SubscriptionState.ACTIVE
-    lifetime = None
     if retroactive:
         state = SubscriptionState.NEW
     if lifetime:
-        lifetime = datetime.datetime.utcnow() + datetime.timedelta(days=lifetime)
+        date_lifetime = datetime.datetime.utcnow() + datetime.timedelta(days=lifetime)
+    else:
+        date_lifetime = None
     new_subscription = models.Subscription(name=name,
                                            filter=filter_,
                                            account=account,
                                            replication_rules=replication_rules,
                                            state=state,
-                                           lifetime=lifetime,
+                                           lifetime=date_lifetime,
                                            retroactive=retroactive,
                                            policyid=priority, comments=comments)
     if keep_history:
@@ -106,11 +124,14 @@ def add_subscription(name, account, filter_, replication_rules, comments, lifeti
            or re.match('.*UniqueViolation.*duplicate key value violates unique constraint.*', error.args[0]):
             raise SubscriptionDuplicate('Subscription \'%s\' owned by \'%s\' already exists!' % (name, account))
         raise RucioException(error.args)
-    return new_subscription.id
+    return str(new_subscription.id)
 
 
 @transactional_session
-def update_subscription(name, account, metadata=None, session=None):
+def update_subscription(name: str,
+                        account: "InternalAccount",
+                        metadata: dict,
+                        session: "Optional[Session]" = None) -> None:
     """
     Updates a subscription
 
@@ -174,7 +195,11 @@ def update_subscription(name, account, metadata=None, session=None):
 
 
 @stream_session
-def list_subscriptions(name=None, account=None, state=None, session=None, logger=logging.log):
+def list_subscriptions(name: "Optional[str]" = None,
+                       account: "Optional[InternalAccount]" = None,
+                       state: "Optional[SubscriptionState]" = None,
+                       session: "Optional[Session]" = None,
+                       logger: "LoggerFunction" = logging.log) -> "Iterator[SubscriptionType]":
     """
     Returns a dictionary with the subscription information :
     Examples: ``{'status': 'INACTIVE/ACTIVE/BROKEN', 'last_modified_date': ...}``
@@ -215,15 +240,15 @@ def list_subscriptions(name=None, account=None, state=None, session=None, logger
         raise SubscriptionNotFound("Subscription for account '%(account)s' named '%(name)s' not found" % locals())
 
 
-def delete_subscription(subscription_id):
+@transactional_session
+def delete_subscription(subscription_id: str, session: "Optional[Session]" = None) -> None:
     """
     Deletes a subscription
 
     :param subscription_id: Subscription identifier
     :type subscription_id:  String
     """
-
-    raise NotImplementedError
+    session.query(models.Subscription).filter_by(id=subscription_id).delete()
 
 
 @stream_session
