@@ -18,6 +18,8 @@ Methods common to different conveyor submitter daemons.
 """
 
 import datetime
+import functools
+import itertools
 import logging
 import re
 from typing import TYPE_CHECKING
@@ -31,21 +33,23 @@ from rucio.core.replica import add_replicas, tombstone_from_delay, update_replic
 from rucio.core.request import set_request_state, queue_requests
 from rucio.core.rse import list_rses
 from rucio.core.rse_expression_parser import parse_expression
+from rucio.core.topology import Topology
+from rucio.core.transfer import list_transfer_admin_accounts, build_transfer_paths
 from rucio.core.vo import list_vos
 from rucio.db.sqla import models
-from rucio.db.sqla.constants import RequestState, RequestType, ReplicaState
+from rucio.db.sqla.constants import RequestState, ReplicaState
 from rucio.db.sqla.session import transactional_session
 from rucio.rse import rsemanager as rsemgr
 
 if TYPE_CHECKING:
-    from typing import Callable, Dict, List, Optional, Set, Tuple, Type, Sequence
+    from typing import Callable, Dict, List, Optional, Set, Tuple, Sequence
     from rucio.core.transfer import DirectTransferDefinition
-    from rucio.transfertool.transfertool import Transfertool, TransferToolBuilder
+    from rucio.transfertool.transfertool import TransferToolBuilder
     from sqlalchemy.orm import Session
 
 
 def next_transfers_to_submit(total_workers=0, worker_number=0, partition_hash_var=None, limit=None, activity=None, older_than=None, rses=None, schemes=None,
-                             failover_schemes=None, filter_transfertool=None, transfertool_classes=None, request_type=RequestType.TRANSFER,
+                             failover_schemes=None, filter_transfertool=None, transfertools=None, request_type=None,
                              ignore_availability=False, logger=logging.log):
     """
     Get next transfers to be submitted; grouped by transfertool which can submit them
@@ -58,15 +62,27 @@ def next_transfers_to_submit(total_workers=0, worker_number=0, partition_hash_va
     :param rses:                  Include RSES.
     :param schemes:               Include schemes.
     :param failover_schemes:      Failover schemes.
-    :param transfertool_classes:  List of transfertool classes which can be used by this submitter
+    :param transfertools:         List of transfertool names which can be used by this submitter
     :param filter_transfertool:   The transfer tool to filter requests on.
-    :param request_type           The type of requests to retrieve (Transfer/Stagein)
+    :param request_type           List of types of requests to retrieve (Transfer/Stagein)
     :param ignore_availability:   Ignore blocklisted RSEs
     :param logger:                Optional decorated logger that can be passed from the calling daemons or servers.
-    :param session:               The database session in use.
     :returns:                     Dict: {TransferToolBuilder: <list of transfer paths (possibly multihop) to be submitted>}
     """
-    candidate_paths, reqs_no_source, reqs_scheme_mismatch, reqs_only_tape_source, reqs_unsupported_transfertool = transfer_core.get_transfer_paths(
+
+    admin_accounts = list_transfer_admin_accounts()
+
+    topology = Topology.create_from_config(ignore_availability=ignore_availability)
+
+    required_source_rse_attrs = None
+    # if filter_transfertool specified, select only the source rses which are configured for this transfertool
+    if filter_transfertool:
+        # if multihop is configured, we want all possible source rses. To allow multi-hopping between transfertools
+        if not topology.multihop_rses:
+            required_source_rse_attrs = transfer_core.TRANSFERTOOL_CLASSES_BY_NAME[filter_transfertool].required_rse_attrs
+
+    # retrieve (from the database) the transfer requests with their possible source replicas
+    requests_with_sources = request_core.list_transfer_requests_and_source_replicas(
         total_workers=total_workers,
         worker_number=worker_number,
         partition_hash_var=partition_hash_var,
@@ -74,23 +90,67 @@ def next_transfers_to_submit(total_workers=0, worker_number=0, partition_hash_va
         activity=activity,
         older_than=older_than,
         rses=rses,
+        request_type=request_type,
+        request_state=RequestState.QUEUED,
+        ignore_availability=ignore_availability,
+        transfertool=filter_transfertool,
+        required_source_rse_attrs=required_source_rse_attrs,
+    )
+
+    # for each source, compute the (possibly multihop) path between it and the transfer destination
+    _build_paths_fnc = functools.partial(
+        build_transfer_paths,
+        topology=topology,
         schemes=schemes,
         failover_schemes=failover_schemes,
-        filter_transfertool=filter_transfertool,
-        active_transfertools={t.external_name for t in transfertool_classes} if transfertool_classes is not None else None,
-        ignore_availability=ignore_availability,
-        request_type=request_type,
+        admin_accounts=admin_accounts,
+        transfertools=transfertools,
         logger=logger,
     )
 
     # Assign paths to be executed by transfertools
     # if the chosen best path is a multihop, create intermediate replicas and the intermediate transfer requests
-    paths_by_transfertool_builder, reqs_no_host = assign_paths_to_transfertool_and_create_hops(
-        candidate_paths,
-        transfertool_classes=transfertool_classes,
+    _assign_path_fnc = functools.partial(
+        assign_paths_to_transfertool_and_create_hops,
+        transfertools=transfertools,
         logger=logger,
     )
 
+    # For requests which had source_rse_id set in the database, start by verifying that it's possible to
+    # use this source to submit it. If it's possible, use it, otherwise, fallback to searching all
+    # possible sources for the request
+    requests_to_load_partially = {}
+    requests_to_load_fully = {}
+    for request_id, rws in requests_with_sources.items():
+        if rws.requested_source and datetime.datetime.utcnow() - datetime.timedelta(hours=6) < rws.requested_at:
+            requests_to_load_partially[request_id] = rws
+        else:
+            requests_to_load_fully[request_id] = rws
+
+    paths_by_transfertool_builder_partial = {}
+    if requests_to_load_partially:
+        candidate_paths, reqs_no_source, reqs_scheme_mismatch, reqs_only_tape_source, reqs_unsupported_transfertool = _build_paths_fnc(
+            requests_with_sources=list(requests_to_load_partially.values()),
+            requested_source_only=True,
+        )
+        paths_by_transfertool_builder_partial, reqs_no_host = _assign_path_fnc(
+            candidate_paths_by_request_id=candidate_paths,
+        )
+        for request_id in itertools.chain(reqs_no_host, reqs_no_source, reqs_scheme_mismatch, reqs_only_tape_source, reqs_unsupported_transfertool):
+            rws = requests_to_load_partially[request_id]
+            logger(logging.INFO, "%s: source_rse_id was set to %s, but cannot be used. Fall-back to full source scan.", request_id, str(rws.requested_source))
+            requests_to_load_fully[request_id] = rws
+
+    candidate_paths, reqs_no_source, reqs_scheme_mismatch, reqs_only_tape_source, reqs_unsupported_transfertool = _build_paths_fnc(
+        requests_with_sources=list(requests_to_load_fully.values())
+    )
+    paths_by_transfertool_builder, reqs_no_host = _assign_path_fnc(
+        candidate_paths_by_request_id=candidate_paths
+    )
+    # Merge the two dicts with submission work into one
+    for tt_builder, paths in paths_by_transfertool_builder_partial.items():
+        paths_by_transfertool_builder.setdefault(tt_builder, []).extend(paths)
+
     if reqs_unsupported_transfertool:
         logger(logging.INFO, "Ignoring request because of unsupported transfertool: %s", reqs_unsupported_transfertool)
     reqs_no_source.update(reqs_no_host)
@@ -109,7 +169,7 @@ def next_transfers_to_submit(total_workers=0, worker_number=0, partition_hash_va
 
 def __assign_to_transfertool(
         transfer_path: "List[DirectTransferDefinition]",
-        transfertool_classes: "List[Type[Transfertool]]",
+        transfertools: "Optional[List[str]]",
         logger: "Callable",
 ) -> "List[Tuple[List[DirectTransferDefinition], Optional[TransferToolBuilder]]]":
     """
@@ -121,7 +181,7 @@ def __assign_to_transfertool(
     For example, for a path A->B->C->D->E, A->B->C may be assigned to transfertool1; while C->D->E to transfertool2.
     This even if transfertool2 could submit the full path in one step without splitting it.
     """
-    if transfertool_classes is None:
+    if transfertools is None:
         return [(transfer_path, None)]
 
     remaining_hops = transfer_path
@@ -129,7 +189,10 @@ def __assign_to_transfertool(
     while remaining_hops:
         tt_builder = None
         assigned_hops = []
-        for transfertool_cls in transfertool_classes:
+        for tt_name in transfertools:
+            transfertool_cls = transfer_core.TRANSFERTOOL_CLASSES_BY_NAME.get(tt_name)
+            if not transfertool_cls:
+                continue
             assigned_hops, tt_builder = transfertool_cls.submission_builder_for_path(remaining_hops, logger=logger)
             if assigned_hops:
                 break
@@ -149,7 +212,7 @@ def __assign_to_transfertool(
 
 def assign_paths_to_transfertool_and_create_hops(
         candidate_paths_by_request_id: "Dict[str: List[DirectTransferDefinition]]",
-        transfertool_classes: "Optional[List[Type[Transfertool]]]" = None,
+        transfertools: "Optional[List[str]]" = None,
         logger: "Callable" = logging.log,
 ) -> "Tuple[Dict[TransferToolBuilder, List[DirectTransferDefinition]], Set[str]]":
     """
@@ -165,7 +228,7 @@ def assign_paths_to_transfertool_and_create_hops(
                 request_id,
                 candidate_paths,
                 default_tombstone_delay=default_tombstone_delay,
-                transfertool_classes=transfertool_classes,
+                transfertools=transfertools,
                 logger=logger,
             )
         except DatabaseException as e:
@@ -190,7 +253,7 @@ def __assign_paths_to_transfertool_and_create_hops(
         request_id: str,
         candidate_paths: "Sequence[List[DirectTransferDefinition]]",
         default_tombstone_delay: int,
-        transfertool_classes: "Optional[List[Type[Transfertool]]]" = None,
+        transfertools: "Optional[List[str]]" = None,
         logger: "Callable" = logging.log,
         session: "Optional[Session]" = None,
 ) -> "Tuple[Optional[List[DirectTransferDefinition]], Optional[TransferToolBuilder]]":
@@ -209,13 +272,13 @@ def __assign_paths_to_transfertool_and_create_hops(
     hops_to_submit = None
     must_skip_submission = False
 
-    tt_assignments = [(transfer_path, __assign_to_transfertool(transfer_path, transfertool_classes, logger=logger))
+    tt_assignments = [(transfer_path, __assign_to_transfertool(transfer_path, transfertools, logger=logger))
                       for transfer_path in candidate_paths]
     # Prioritize the paths which need less transfertool transitions.
     # Ideally, the entire path should be submitted to a single transfertool
     for transfer_path, tt_assignment in sorted(tt_assignments, key=lambda t: len(t[1])):
         if not tt_assignment:
-            logger(logging.INFO, '%s: None of the transfertools can submit the request: %s', request_id, [c.__name__ for c in transfertool_classes])
+            logger(logging.INFO, '%s: None of the transfertools can submit the request: %s', request_id, transfertools)
             continue
 
         # Set the 'transfertool' field on the intermediate hops which should be created in the database
