@@ -48,9 +48,9 @@ def run_once(bulk, group_bulk, rse_ids, scheme, failover_scheme, transfertool_kw
         activity=activity,
         rses=rse_ids,
         schemes=scheme,
-        transfertool_classes=[FTS3Transfertool],
+        transfertools=[FTS3Transfertool.external_name],
         older_than=None,
-        request_type=RequestType.STAGEIN,
+        request_type=[RequestType.STAGEIN],
         logger=logger,
     )
     total_transfers = len(list(hop for paths in transfers.values() for path in paths for hop in path))
