@@ -255,12 +255,18 @@ def get_subscriptions(logger: "Callable" = logging.log) -> List[Dict]:
     except SubscriptionNotFound as error:
         logger(logging.WARNING, "No subscriptions defined: %s" % (str(error)))
         return []
+    except TypeError as error:
+        logger(
+            logging.ERROR,
+            "Failed to parse subscription: %s" % (str(error)),
+        )
+        raise error
     except Exception as error:
         logger(
             logging.ERROR,
             "Failed to get list of new DIDs or subscriptions: %s" % (str(error)),
         )
-        return []
+        raise error
     return subscriptions
 
 
@@ -319,6 +325,19 @@ def __is_matching_subscription(subscription, did, metadata):
                     break
             if not match_did_type:
                 return False
+        elif key in ["min_avg_file_size", "max_avg_file_size"]:
+            length = metadata["length"]
+            size = metadata["bytes"]
+            if length and size:
+                avg_file_size = size / length
+                if key == "min_avg_file_size" and avg_file_size < values:
+                    return False
+                if key == "max_avg_file_size" and avg_file_size > values:
+                    return False
+            else:
+                # If the DID is evaluated at the creation, length and bytes are not set yet
+                # In that case, just ignore min_avg_file_size and max_avg_file_size filter
+                continue
         else:
             if not isinstance(values, list):
                 values = [
