@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2016-2020 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,11 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2016
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2017
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2019-2020
 
 ''' new table for lifetime model exceptions '''
 
@@ -47,11 +42,17 @@ def upgrade():
                      sa.Column('id', GUID()),
                      sa.Column('scope', sa.String(25)),
                      sa.Column('name', sa.String(255)),
-                     sa.Column('did_type', sa.Enum(DIDType, name='LIFETIME_EXCEPT_TYPE_CHK', values_callable=lambda obj: [e.value for e in obj])),
+                     sa.Column('did_type', sa.Enum(DIDType,
+                                                   name='LIFETIME_EXCEPT_TYPE_CHK',
+                                                   create_constraint=True,
+                                                   values_callable=lambda obj: [e.value for e in obj])),
                      sa.Column('account', sa.String(25)),
                      sa.Column('comments', sa.String(4000)),
                      sa.Column('pattern', sa.String(255)),
-                     sa.Column('state', sa.Enum(LifetimeExceptionsState, name='LIFETIME_EXCEPT_STATE_CHK', values_callable=lambda obj: [e.value for e in obj])),
+                     sa.Column('state', sa.Enum(LifetimeExceptionsState,
+                                                name='LIFETIME_EXCEPT_STATE_CHK',
+                                                create_constraint=True,
+                                                values_callable=lambda obj: [e.value for e in obj])),
                      sa.Column('created_at', sa.DateTime, default=datetime.datetime.utcnow),
                      sa.Column('updated_at', sa.DateTime, default=datetime.datetime.utcnow, onupdate=datetime.datetime.utcnow),
                      sa.Column('expires_at', sa.DateTime))
