@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2022 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,32 +12,11 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2015-2017
-# - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2019
-# - Wen Guan <wen.guan@cern.ch>, 2015
-# - Martin Barisits <martin.barisits@cern.ch>, 2015-2020
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2016-2021
-# - Mario Lassnig <mario.lassnig@cern.ch>, 2017-2020
-# - asket <asket.agarwal96@gmail.com>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
-# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
-# - Ruturaj Gujar <ruturaj.gujar23@gmail.com>, 2019
-# - Jaroslav Guenther <jaroslav.guenther@cern.ch>, 2019-2020
-# - Eli Chadwick <eli.chadwick@stfc.ac.uk>, 2020
-# - James Perry <j.perry@epcc.ed.ac.uk>, 2020
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
-# - Matt Snyder <msnyder@bnl.gov>, 2021
-# - joaquinbogado <joaquinbogado@gmail.com>, 2021
-# - Ilija Vukotic <ivukotic@uchicago.edu>, 2021
-# - Radu Carpa <radu.carpa@cern.ch>, 2022
 
 import datetime
 import sys
 import uuid
 
-from six import iteritems
 from sqlalchemy import BigInteger, Boolean, Column, DateTime, Enum, Float, Integer, SmallInteger, String as _String, Text, event, UniqueConstraint
 from sqlalchemy.engine import Engine
 from sqlalchemy.ext.compiler import compiles
@@ -261,7 +240,7 @@ class ModelBase(object):
 
     def update(self, values, flush=True, session=None):
         """dict.update() behaviour."""
-        for k, v in iteritems(values):
+        for k, v in values.items():
             self[k] = v
         if session and flush:
             session.flush()
@@ -781,7 +760,10 @@ class RSE(BASE, SoftModelBase):
     ASN = Column(String(255))
     longitude = Column(Float())
     latitude = Column(Float())
-    availability = Column(Integer, server_default='7')
+    availability = Column(Integer, server_default='7')  # Deprecated, will be removedx
+    availability_read = Column(Boolean, default=True)
+    availability_write = Column(Boolean, default=True)
+    availability_delete = Column(Boolean, default=True)
     usage = relationship("RSEUsage", order_by="RSEUsage.rse_id", backref="rses")
     qos_class = Column(String(64))
     _table_args = (PrimaryKeyConstraint('id', name='RSES_PK'),
@@ -879,7 +861,6 @@ class RSEProtocols(BASE, ModelBase):
     read_wan = Column(Integer, server_default='0')  # if no value is provided, 0 i.e. not supported is assumed as default value
     write_wan = Column(Integer, server_default='0')  # if no value is provided, 0 i.e. not supported is assumed as default value
     delete_wan = Column(Integer, server_default='0')  # if no value is provided, 0 i.e. not supported is assumed as default value
-    third_party_copy = Column(Integer, server_default='0')  # if no value is provided, 0 i.e. not supported is assumed as default value
     third_party_copy_read = Column(Integer, server_default='0')  # if no value is provided, 0 i.e. not supported is assumed as default value
     third_party_copy_write = Column(Integer, server_default='0')  # if no value is provided, 0 i.e. not supported is assumed as default value
     extended_attributes = Column(String(4000), nullable=True)
@@ -1498,7 +1479,7 @@ class Token(BASE, ModelBase):
     __tablename__ = 'tokens'
     token = Column(String(3072))  # account-identity-appid-uuid -> max length: (+ 30 1 255 1 32 1 32)
     account = Column(InternalAccountString(get_schema_value('ACCOUNT_LENGTH')))
-    refresh_token = Column(String(315), default=None)
+    refresh_token = Column(String(3072), default=None)
     refresh = Column(Boolean(name='TOKENS_REFRESH_CHK', create_constraint=True),
                      default=False)
     refresh_start = Column(DateTime, default=None)
