@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2015-2021 CERN
+# Copyright European Organization for Nuclear Research (CERN) since 2012
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -12,14 +12,6 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#
-# Authors:
-# - Fernando López <felopez@cern.ch>, 2015
-# - Martin Barisits <martin.barisits@cern.ch>, 2017
-# - Vincent Garonne <vincent.garonne@cern.ch>, 2018
-# - Dimitrios Christidis <dimitrios.christidis@cern.ch>, 2018
-# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2019
-# - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 
 import sys
 from datetime import datetime
@@ -28,12 +20,7 @@ import pytest
 
 from rucio.daemons.auditor import srmdumps
 
-try:
-    # PY2
-    from ConfigParser import ConfigParser
-except ImportError:
-    # PY3
-    from configparser import ConfigParser
+from configparser import ConfigParser
 
 if sys.version_info >= (3, 3):
     from unittest import mock
