@@ -26,7 +26,7 @@ import xmltodict
 from werkzeug.datastructures import MultiDict
 
 from rucio.client.ruleclient import RuleClient
-from rucio.common.exception import (DataIdentifierNotFound, AccessDenied, RucioException,
+from rucio.common.exception import (DataIdentifierNotFound, AccessDenied, RSEProtocolPriorityError, RucioException,
                                     ReplicaIsLocked, ReplicaNotFound, ScopeNotFound,
                                     DatabaseException, InputValidationError)
 from rucio.common.schema import get_schema_value
@@ -409,6 +409,27 @@ class TestReplicaCore:
         replicas = list(replica_client.list_replicas([{'scope': mock_scope.external, 'name': name}]))
         assert 'http://' in list(replicas[0]['pfns'].keys())[0]
 
+        with pytest.raises(RSEProtocolPriorityError):
+            # check that the appropriate exception is raised when an invalid setting is passed
+            add_protocol(rse_id, {'scheme': 'http',
+                                  'hostname': 'http.aperture.com',
+                                  'port': 80,
+                                  'prefix': '//test/chamber/',
+                                  'impl': 'rucio.rse.protocols.gfal.Default',
+                                  'domains': {
+                                      'lan': {'read': None, 'write': 1, 'delete': 1},  # None should be int >= 0
+                                      'wan': {'read': 1, 'write': 1, 'delete': 1}}})
+        with pytest.raises(RSEProtocolPriorityError):
+            # check that the appropriate exception is raised when an invalid setting is passed
+            add_protocol(rse_id, {'scheme': 'http',
+                                  'hostname': 'http.aperture.com',
+                                  'port': 80,
+                                  'prefix': '//test/chamber/',
+                                  'impl': 'rucio.rse.protocols.gfal.Default',
+                                  'domains': {
+                                      'lan': {'read': 1, 'write': 1, 'delete': 1},  # None should be int >= 0
+                                      'wan': {'read': None, 'write': 1, 'delete': 1}}})
+
     def test_replica_no_site(self, rse_factory, mock_scope, root_account, replica_client):
         """ REPLICA (CORE): Test listing replicas without site attribute """
 
@@ -513,12 +534,12 @@ class TestReplicaCore:
         add_rse_attribute(rse_id=rse2_id, key='tombstone_delay', value=tombstone_delay)
 
         # Will use the default tombstone delay
-        did1 = did_factory.random_did()
+        did1 = did_factory.random_file_did()
         add_replica(rse1_id, bytes_=4, account=root_account, **did1)
         assert get_replica(rse1_id, **did1)['tombstone'] is None
 
         # Will use the configured value on the RSE
-        did2 = did_factory.random_did()
+        did2 = did_factory.random_file_did()
         add_replica(rse2_id, bytes_=4, account=root_account, **did2)
         tombstone = get_replica(rse2_id, **did2)['tombstone']
         expected_tombstone = datetime.utcnow() + timedelta(seconds=tombstone_delay)
@@ -610,6 +631,64 @@ class TestReplicaCore:
         assert cov[rse2_id] == 300
         assert cov[rse3_id] == 500
 
+    @pytest.mark.parametrize("caches_mock", [{"caches_to_mock": [
+        'rucio.core.rse_expression_parser.REGION',
+    ]}], indirect=True)
+    @pytest.mark.parametrize("file_config_mock", [
+        # Run test twice: with, and without, temp tables
+        {"overrides": [('core', 'use_temp_tables', 'True')]},
+        {"overrides": [('core', 'use_temp_tables', 'False')]},
+    ], indirect=True)
+    def test_list_replicas_rse_filter(self, rse_factory, mock_scope, root_account, file_config_mock, caches_mock):
+        """ REPLICA (CORE): test rse filter for list replicas """
+        nbrses = 10
+        nbfiles = 2
+        rses = [rse_factory.make_mock_rse() for _ in range(nbrses)]
+        rses_group1 = set()
+        rses_group2 = set()
+        for i, (_, rse_id) in enumerate(rses):
+            # Create two overlapping groups of rses: first 75% and last 75%
+            if i < nbrses * 3 // 4:
+                rses_group1.add(rse_id)
+                add_rse_attribute(rse_id, 'group1', 'true')
+            if i > nbrses * 1 // 4:
+                rses_group2.add(rse_id)
+                add_rse_attribute(rse_id, 'group2', 'true')
+
+        files = [{'scope': mock_scope, 'name': did_name_generator('file'), 'bytes': 1, 'adler32': '0cc737eb', 'meta': {'events': 10}} for _ in range(nbfiles)]
+
+        # Add one file on all rses
+        file1 = files[0]
+        did1 = {'scope': file1['scope'], 'name': file1['name']}
+        for _, rse_id in rses:
+            add_replicas(rse_id=rse_id, files=[file1], account=root_account, ignore_availability=True)
+        [replica] = list(list_replicas([did1]))
+        assert len(replica['pfns']) == nbrses
+        [replica] = list(list_replicas([did1], rse_expression='group1=true|group2=true'))
+        assert len(replica['pfns']) == nbrses
+        [replica] = list(list_replicas([did1], rse_expression='group1=true&group2=true'))
+        assert len(replica['pfns']) == len(rses_group1.intersection(rses_group2))
+        [replica] = list(list_replicas([did1], rse_expression='group1=true'))
+        assert len(replica['pfns']) == len(rses_group1)
+        [replica] = list(list_replicas([did1], rse_expression='group2=true'))
+        assert len(replica['pfns']) == len(rses_group2)
+
+        # Add another file to one rse in group1 and one rse in group2
+        file2 = files[1]
+        did2 = {'scope': file2['scope'], 'name': file2['name']}
+        for _, rse_id in (rses[0], rses[-1]):
+            add_replicas(rse_id=rse_id, files=[file2], account=root_account, ignore_availability=True)
+        [replica] = list(list_replicas([did2]))
+        assert len(replica['pfns']) == 2
+        [replica] = list(list_replicas([did2], rse_expression='group1=true|group2=true'))
+        assert len(replica['pfns']) == 2
+        [replica] = list(list_replicas([did2], rse_expression='group1=true&group2=true'))
+        assert len(replica['pfns']) == 0
+        [replica] = list(list_replicas([did2], rse_expression='group1=true'))
+        assert len(replica['pfns']) == 1
+        [replica] = list(list_replicas([did2], rse_expression='group2=true'))
+        assert len(replica['pfns']) == 1
+
 
 @pytest.mark.parametrize("core_config_mock", [{"table_content": [
     ('reaper', 'remove_open_did', True)
@@ -734,7 +813,7 @@ def test_client_access_denied_on_delete_replicas(rse_factory, mock_scope, replic
 def test_client_list_replicas_on_did_without_replicas(rse_factory, did_factory, replica_client, did_client, root_account):
     """ REPLICA (CLIENT): dids of type FILE, but without replicas, must be listed with empty pfns and rses"""
     rse, _ = rse_factory.make_posix_rse()
-    file = did_factory.random_did()
+    file = did_factory.random_file_did()
     dataset = did_factory.make_dataset()
     container = did_factory.make_container()
 
