@@ -19,7 +19,7 @@ import functools
 import logging
 import traceback
 import uuid
-from typing import Any, Callable, Tuple, TYPE_CHECKING
+from typing import Any, Callable, Dict, Tuple, TYPE_CHECKING
 from urllib.parse import urlparse
 
 import requests
@@ -44,6 +44,7 @@ from rucio.db.sqla.constants import RequestState
 
 if TYPE_CHECKING:
     from rucio.core.transfer import DirectTransferDefinition
+    from rucio.core.rse import RseData
 
 logging.getLogger("requests").setLevel(logging.CRITICAL)
 disable_warnings()
@@ -67,7 +68,6 @@ BULK_QUERY_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_bulk_query', sta
 QUERY_DETAILS_COUNTER = MultiCounter(prom='rucio_transfertool_fts3_query_details', statsd='transfertool.fts3.{host}.query_details.{state}',
                                      documentation='Number of detailed status queries', labelnames=('state', 'host'))
 
-
 ALLOW_USER_OIDC_TOKENS = config_get_bool('conveyor', 'allow_user_oidc_tokens', False, False)
 REQUEST_OIDC_SCOPE = config_get('conveyor', 'request_oidc_scope', False, 'fts:submit-transfer')
 REQUEST_OIDC_AUDIENCE = config_get('conveyor', 'request_oidc_audience', False, 'fts:example')
@@ -446,7 +446,8 @@ class Fts3TransferStatusReport(TransferStatusReport):
         return False
 
     @classmethod
-    def _is_recoverable_fts_overwrite_error(cls, request, reason, file_metadata):
+    def _is_recoverable_fts_overwrite_error(cls, request: Dict[str, Any], reason: str,
+                                            file_metadata: Dict[str, Any]) -> bool:
         """
         Verify the special case when FTS cannot copy a file because destination exists and overwrite is disabled,
         but the destination file is actually correct.
@@ -460,12 +461,18 @@ class Fts3TransferStatusReport(TransferStatusReport):
             return False
         dst_file = file_metadata.get('dst_file', {})
         dst_type = file_metadata.get('dst_type', None)
+        record_counter('transfertool.fts3.overwrite.check.{rsetype}.{rse}',
+                       labels={'rse': file_metadata["dst_rse"], 'rsetype': dst_type})
+
         if 'Destination file exists and overwrite is not enabled' in (reason or ''):
             if cls._dst_file_set_and_file_correct(request, dst_file):
-                if dst_file.get('file_on_tape'):
-                    return True
-                elif dst_type == 'DISK':
+                if dst_type == 'DISK' or dst_file.get('file_on_tape'):
+                    record_counter('transfertool.fts3.overwrite.ok.{rsetype}.{rse}',
+                                   labels={'rse': file_metadata["dst_rse"], 'rsetype': dst_type})
                     return True
+
+        record_counter('transfertool.fts3.overwrite.fail.{rsetype}.{rse}',
+                       labels={'rse': file_metadata["dst_rse"], 'rsetype': dst_type})
         return False
 
 
@@ -555,9 +562,11 @@ class FTS3ApiTransferStatusReport(Fts3TransferStatusReport):
         self._multi_sources = str(job_response['job_metadata'].get('multi_sources', '')).lower() == str('true')
         self._src_url = file_response.get('source_surl', None)
         self._dst_url = file_response.get('dest_surl', None)
+        self.logger = logging.log
 
     def initialize(self, session, logger=logging.log):
 
+        self.logger = logger
         job_response = self.job_response
         file_response = self.file_response
         request_id = self.request_id
@@ -631,6 +640,7 @@ class FTS3Transfertool(Transfertool):
     """
 
     external_name = 'fts3'
+    required_rse_attrs = ('fts', )
 
     def __init__(self, external_host, oidc_account=None, vo=None, group_bulk=1, group_policy='rule', source_strategy=None,
                  max_time_in_queue=None, bring_online=43200, default_lifetime=172800, archive_timeout_override=None,
@@ -684,6 +694,28 @@ class FTS3Transfertool(Transfertool):
             self.cert = None
             self.verify = True  # True is the default setting of a requests.* method
 
+    @classmethod
+    def _pick_fts_servers(cls, source_rse: "RseData", dest_rse: "RseData"):
+        """
+        Pick fts servers to use for submission between the two given rse
+        """
+        source_servers = source_rse.attributes.get('fts', None)
+        dest_servers = dest_rse.attributes.get('fts', None)
+        if source_servers is None or dest_servers is None:
+            return None
+
+        servers_to_use = dest_servers
+        if source_rse.attributes.get('sign_url', None) == 'gcs':
+            servers_to_use = source_servers
+
+        return servers_to_use.split(',')
+
+    @classmethod
+    def can_perform_transfer(cls, source_rse: "RseData", dest_rse: "RseData"):
+        if cls._pick_fts_servers(source_rse, dest_rse):
+            return True
+        return False
+
     @classmethod
     def submission_builder_for_path(cls, transfer_path, logger=logging.log):
         vo = None
@@ -693,13 +725,8 @@ class FTS3Transfertool(Transfertool):
         sub_path = []
         fts_hosts = []
         for hop in transfer_path:
-            src_and_dst_have_fts_attribute = hop.src.rse.attributes.get('fts', None) is not None and hop.dst.rse.attributes.get('fts', None) is not None
-            hosts = hop.dst.rse.attributes.get('fts', None)
-            if hop.src.rse.attributes.get('sign_url', None) == 'gcs':
-                hosts = hop.src.rse.attributes.get('fts', None)
-            hosts = hosts.split(",") if hosts else []
-
-            if src_and_dst_have_fts_attribute and hosts:
+            hosts = cls._pick_fts_servers(hop.src.rse, hop.dst.rse)
+            if hosts:
                 fts_hosts = hosts
                 sub_path.append(hop)
             else:
