@@ -16,6 +16,7 @@
 import base64
 import logging
 import time
+import json
 from re import search
 from typing import TYPE_CHECKING
 
@@ -27,9 +28,10 @@ from rucio.api.authentication import get_auth_token_user_pass, get_auth_token_gs
     get_auth_token_ssh, get_ssh_challenge_token, validate_auth_token, get_auth_oidc, redirect_auth_oidc, \
     get_token_oidc, refresh_cli_auth_token, get_auth_token_saml
 from rucio.common.config import config_get
-from rucio.common.exception import AccessDenied, IdentityError, CannotAuthenticate, CannotAuthorize
+from rucio.common.exception import AccessDenied, IdentityError, IdentityNotFound, CannotAuthenticate, CannotAuthorize
 from rucio.common.extra import import_extras
 from rucio.common.utils import date_to_str
+from rucio.api.identity import list_accounts_for_identity, get_default_account, verify_identity
 from rucio.web.rest.flaskapi.v1.common import check_accept_header_wrapper_flask, error_headers, \
     extract_vo, generate_http_error_flask, ErrorHandlingMethodView
 
@@ -55,19 +57,40 @@ class UserPass(ErrorHandlingMethodView):
         headers['Access-Control-Allow-Headers'] = request.environ.get('HTTP_ACCESS_CONTROL_REQUEST_HEADERS')
         headers['Access-Control-Allow-Methods'] = '*'
         headers['Access-Control-Allow-Credentials'] = 'true'
-        headers['Access-Control-Expose-Headers'] = 'X-Rucio-Auth-Token'
+        headers['Access-Control-Expose-Headers'] = 'X-Rucio-Auth-Token, X-Rucio-Auth-Account, X-Rucio-Auth-Accounts'
         return headers
 
     def options(self):
         """
-        Allow cross-site scripting. Explicit for Authentication.
-
-        :resheader Access-Control-Allow-Origin:
-        :resheader Access-Control-Allow-Headers:
-        :resheader Access-Control-Allow-Methods:
-        :resheader Access-Control-Allow-Credentials:
-        :resheader Access-Control-Expose-Headers:
-        :status 200: OK
+        ---
+        summary: UserPass Allow cross-site scripting
+        description: UserPass Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+              Access-Control-Expose-Headers:
+                schema:
+                  type: string
+                  enum: ['X-Rucio-Auth-Token']
+          404:
+            description: Not found
         """
 
         return '', 200, self.get_headers()
@@ -75,23 +98,79 @@ class UserPass(ErrorHandlingMethodView):
     @check_accept_header_wrapper_flask(['application/octet-stream'])
     def get(self):
         """
-        Authenticate a Rucio account temporarily via username and password.
+        ---
+        summary: UserPass
+        description: Authenticate a Rucio account temporarily via username and password.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Rucio-Account
+          in: header
+          description: Account identifier as a string.
+          schema:
+            type: string
+          required: true
+        - name: X-Rucio-Username
+          in: header
+          description: Username as a string.
+          schema:
+            type: string
+          required: true
+        - name: X-Rucio-Password
+          in: header
+          description: password as a text-plain string.
+          schema:
+            type: string
+          required: true
+        - name: X-Rucio-AppID
+          in: header
+          description: Application identifier as a string.
+          schema:
+            type: string
+        - name: X-Forwarded-For
+          in: header
+          description: The forward ip address.
+          schema:
+            type: string
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+              Access-Control-Expose-Headers:
+                schema:
+                  type: string
+              X-Rucio-Auth-Token:
+                schema:
+                  type: string
+              X-Rucio-Auth-Token-Expires:
+                schema:
+                  type: string
+              X-Rucio-Auth-Account:
+                schema:
+                  type: string
+                description: The rucio account used for authentication
+          206:
+            description: Partial content containing X-Rucio-Auth-Accounts header
+            headers:
+              X-Rucio-Auth-Accounts:
+                schema:
+                  type: string
+                description: The rucio accounts corresponding to the provided identity as a csv string
 
-        .. :quickref: UserPass; Authenticate with username/password
-
-        :reqheader X-Rucio-VO: VO name as a string (Multi-VO Only)
-        :reqheader X-Rucio-Account: Account identifier as a string.
-        :reqheader X-Rucio-Username: Username as a string.
-        :reqheader X-Rucio-Password: password as a text-plain string.
-        :reqheader X-Rucio-AppID: Application identifier as a string.
-        :resheader Access-Control-Allow-Origin:
-        :resheader Access-Control-Allow-Headers:
-        :resheader Access-Control-Allow-Methods:
-        :resheader Access-Control-Allow-Credentials:
-        :resheader Access-Control-Expose-Headers:
-        :resheader X-Rucio-Auth-Token: The authentication token
-        :status 200: Successfully authenticated
-        :status 404: Invalid credentials
+          401:
+            description: Cannot authenticate
         """
         headers = self.get_headers()
 
@@ -107,17 +186,37 @@ class UserPass(ErrorHandlingMethodView):
         appid = request.headers.get('X-Rucio-AppID', default='unknown')
         ip = request.headers.get('X-Forwarded-For', default=request.remote_addr)
 
-        if not account or not username or not password:
+        if not username or not password:
             return generate_http_error_flask(401, CannotAuthenticate.__name__, 'Cannot authenticate without passing all required arguments', headers=headers)
 
         try:
-            result = get_auth_token_user_pass(account, username, password, appid, ip, vo=vo)
+            verify_identity(identity_key=username, id_type='USERPASS', password=password)
+        except IdentityNotFound:
+            return generate_http_error_flask(401, IdentityNotFound.__name__, 'Cannot authenticate. Username/Password pair does not exist.', headers=headers)
+        except IdentityError:
+            return generate_http_error_flask(401, IdentityError.__name__, 'Cannot authenticate. The identity does not exist.', headers=headers)
+
+        if not account:
+            accounts = list_accounts_for_identity(identity_key=username, id_type='USERPASS')
+            if len(accounts) == 0 or accounts is None:
+                return generate_http_error_flask(401, CannotAuthenticate.__name__, 'Cannot authenticate with provided username or password. Identity is not mapped to any accounts.', headers=headers)
+            if len(accounts) > 1:
+                try:
+                    account = get_default_account(identity_key=username, id_type='USERPASS')
+                except IdentityError:
+                    headers['X-Rucio-Auth-Accounts'] = ','.join(accounts)
+                    return json.dumps(accounts), 206, headers
+            else:
+                account = accounts[0]
+        account_name = account.external if type(account) is not str else account
+        try:
+            result = get_auth_token_user_pass(account_name, username, password, appid, ip, vo=vo)
         except AccessDenied:
             return generate_http_error_flask(401, CannotAuthenticate.__name__, f'Cannot authenticate to account {account} with given credentials', headers=headers)
 
         if not result:
             return generate_http_error_flask(401, CannotAuthenticate.__name__, f'Cannot authenticate to account {account} with given credentials', headers=headers)
-
+        headers['X-Rucio-Auth-Account'] = account_name
         headers['X-Rucio-Auth-Token'] = result['token']
         headers['X-Rucio-Auth-Token-Expires'] = date_to_str(result['expires_at'])
         return '', 200, headers
@@ -139,9 +238,31 @@ class OIDC(ErrorHandlingMethodView):
 
     def options(self):
         """
-        Allow cross-site scripting. Explicit for Authentication.
-
-        :status 200: OK
+        ---
+        summary: OIDC Allow cross-site scripting
+        description: OIDC Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+          404:
+            description: Not found
         """
 
         return '', 200, self.get_headers()
@@ -149,12 +270,55 @@ class OIDC(ErrorHandlingMethodView):
     @check_accept_header_wrapper_flask(['application/octet-stream'])
     def get(self):
         """
-
-        .. :quickref: OIDC; Authenticate with OIDC
-
-        :status 200: OK
-        :status 401: Unauthorized
-        :resheader X-Rucio-OIDC-Auth-URL: User & Rucio OIDC Client specific Authorization URL
+        ---
+        summary: OIDC
+        description: Authenticate a Rucio account via OIDC.
+        tags:
+          - Auth
+        parameters:
+        - name: HTTP_X_RUCIO_ACCOUNT
+          in: header
+          description: Account identifier as a string.
+          schema:
+            type: string
+        - name: HTTP_X_RUCIO_CLIENT_AUTHORIZE_SCOPE
+          in: header
+          schema:
+            type: string
+        - name: HTTP_X_RUCIO_CLIENT_AUTHORIZE_AUDIENCE
+          in: header
+          schema:
+            type: string
+        - name: HTTP_X_RUCIO_CLIENT_AUTHORIZE_AUTO
+          in: header
+          schema:
+            type: string
+        - name: HTTP_X_RUCIO_CLIENT_AUTHORIZE_ISSUER
+          in: header
+          schema:
+            type: string
+        - name: HTTP_X_RUCIO_CLIENT_AUTHORIZE_POLLING
+          in: header
+          schema:
+            type: string
+        - name: HTTP_X_RUCIO_CLIENT_AUTHORIZE_REFRESH_LIFETIME
+          in: header
+          schema:
+            type: string
+        - name: X-Forwarded-For
+          in: header
+          schema:
+            type: string
+        responses:
+          200:
+            description: OK
+            headers:
+              X-Rucio-OIDC-Auth-URL:
+                description: User & Rucio OIDC Client specific Authorization URL
+                schema:
+                  type: string
+          401:
+            description: Cannot authenticate
         """
         headers = self.get_headers()
 
@@ -221,21 +385,63 @@ class RedirectOIDC(ErrorHandlingMethodView):
 
     def options(self):
         """
-        Allow cross-site scripting. Explicit for Authentication.
-
-        :status 200: OK
+        ---
+        summary: RedirectOIDC Allow cross-site scripting
+        description: RedirectOIDC Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
     @check_accept_header_wrapper_flask(['application/octet-stream', 'text/html'])
     def get(self):
         """
-        .. :quickref: OIDC;
-
-        :status 200: OK
-        :status 303: Redirect
-        :status 401: Unauthorized
-        :resheader X-Rucio-Auth-Token: The authentication token
+        ---
+        summary: RedirectOIDC
+        description: Authenticate a Rucio account via RedirectOIDC.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Rucio-Client-Fetch-Token
+          in: header
+          schema:
+            type: string
+        responses:
+          200:
+            description: OK
+            headers:
+              X-Rucio-Auth-Token:
+                description: The authentication token
+                schema:
+                  type: string
+              Content-Type:
+                schema:
+                  type: string
+                  enum: ['application/octet-stream']
+          303:
+            description: Redirect
+          401:
+            description: Cannot authenticate
         """
         headers = self.get_headers()
 
@@ -294,20 +500,52 @@ class CodeOIDC(ErrorHandlingMethodView):
 
     def options(self):
         """
-        HTTP Success:
-            200 OK
-
-        Allow cross-site scripting. Explicit for Authentication.
+        ---
+        summary: CodeOIDC Allow cross-site scripting
+        description: CodeOIDC Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
     @check_accept_header_wrapper_flask(['application/octet-stream', 'text/html'])
     def get(self):
         """
-        .. :quickref: OIDC;
-
-        :status 200: OK
-        :status 401: Unauthorized
+        ---
+        summary: CodeOIDC
+        description: Authenticate a Rucio account via CodeOIDC.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Forwarded-For
+          in: header
+          schema:
+            type: string
+        responses:
+          200:
+            description: OK
+          400:
+            description: Invalid request
         """
         headers = self.get_headers()
 
@@ -359,21 +597,61 @@ class TokenOIDC(ErrorHandlingMethodView):
 
     def options(self):
         """
-        Allow cross-site scripting. Explicit for Authentication.
-
-        :status 200: OK
+        ---
+        summary: TokenOIDC Allow cross-site scripting
+        description: TokenOIDC Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
     @check_accept_header_wrapper_flask(['application/octet-stream'])
     def get(self):
         """
-        .. :quickref: OIDC;
-
-        :status 200: OK
-        :status 401: Unauthorized
-        :resheader X-Rucio-Auth-Token: The authentication token
-        :resheader X-Rucio-Auth-Token-Expires: The time when the token expires
+        ---
+        summary: TokenOIDC
+        description: Authenticate a Rucio account via TokenOIDC.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Forwarded-For
+          in: header
+          schema:
+            type: string
+        responses:
+          200:
+            description: OK
+            headers:
+              X-Rucio-Auth-Token:
+                description: The authentication token
+                schema:
+                  type: string
+              X-Rucio-Auth-Token-Expires:
+                description: The time when the token expires
+                schema:
+                  type: string
+          401:
+            description: Cannot authenticate
         """
         headers = self.get_headers()
 
@@ -434,20 +712,71 @@ class RefreshOIDC(ErrorHandlingMethodView):
 
     def options(self):
         """
-        Allow cross-site scripting. Explicit for Authentication.
-
-        :status 200: OK
+        ---
+        summary: RefreshOIDC Allow cross-site scripting
+        description: RefreshOIDC Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+              Access-Control-Expose-Headers:
+                schema:
+                  type: string
+                  enum: ['X-Rucio-Auth-Token']
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
     @check_accept_header_wrapper_flask(['application/octet-stream'])
     def get(self):
         """
-        .. :quickref: OIDC;
-
-        :status 200: OK
-        :status 401: Unauthorized
-        :resheader X-Rucio-Auth-Token: The authentication token
+        ---
+        summary: RefreshOIDC
+        description: Authenticate a Rucio account via RefreshOIDC.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Rucio-Account
+          in: header
+          schema:
+            type: string
+          required: true
+        - name: X-Rucio-Auth-Token
+          in: header
+          schema:
+            type: string
+          required: true
+        responses:
+          200:
+            description: OK
+            headers:
+              X-Rucio-Auth-Token:
+                description: The authentication token
+                schema:
+                  type: string
+              X-Rucio-Auth-Token-Expires:
+                description: The time when the token expires
+                schema:
+                  type: string
+          401:
+            description: Cannot authenticate
         """
         headers = self.get_headers()
 
@@ -492,32 +821,79 @@ class GSS(ErrorHandlingMethodView):
 
     def options(self):
         """
-        HTTP Success:
-            200 OK
-
-        Allow cross-site scripting. Explicit for Authentication.
+        ---
+        summary: GSS Allow cross-site scripting
+        description: GSS Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+              Access-Control-Expose-Headers:
+                schema:
+                  type: string
+                  enum: ['X-Rucio-Auth-Token']
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
     @check_accept_header_wrapper_flask(['application/octet-stream'])
     def get(self):
         """
-        Authenticate a Rucio account temporarily via a GSS token.
-
-        .. :quickref: GSS; Authenticate with GSS token
-
-        :reqheader Rucio-VO: VO name as a string (Multi-VO only).
-        :reqheader Rucio-Account: Account identifier as a string.
-        :reqheader Rucio-AppID: Application identifier as a string.
-        :reqheader SavedCredentials: Apache mod_auth_kerb SavedCredentials.
-        :resheader Access-Control-Allow-Origin:
-        :resheader Access-Control-Allow-Headers:
-        :resheader Access-Control-Allow-Methods:
-        :resheader Access-Control-Allow-Credentials:
-        :resheader Access-Control-Expose-Headers:
-        :resheader X-Rucio-Auth-Token: The authentication token
-        :status 200: Successfully authenticated
-        :status 404: Invalid credentials
+        ---
+        summary: GSS
+        description: Authenticate a Rucio account via GSS.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Rucio-Account
+          in: header
+          schema:
+            type: string
+          required: true
+        - name: REMOTE_USER
+          in: header
+          schema:
+            type: string
+          required: true
+        - name: X-Rucio-AppID
+          in: header
+          schema:
+            type: string
+        - name: X-Forwarded-For
+          in: header
+          schema:
+            type: string
+        responses:
+          200:
+            description: OK
+            headers:
+              X-Rucio-Auth-Token:
+                description: The authentication token
+                schema:
+                  type: string
+              X-Rucio-Auth-Token-Expires:
+                description: The time when the token expires
+                schema:
+                  type: string
+          401:
+            description: Cannot authenticate
         """
 
         headers = self.get_headers()
@@ -572,32 +948,74 @@ class x509(ErrorHandlingMethodView):
 
     def options(self):
         """
-        HTTP Success:
-            200 OK
-
-        Allow cross-site scripting. Explicit for Authentication.
+        ---
+        summary: x509 Allow cross-site scripting
+        description: x509 Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+              Access-Control-Expose-Headers:
+                schema:
+                  type: string
+                  enum: ['X-Rucio-Auth-Token']
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
     @check_accept_header_wrapper_flask(['application/octet-stream'])
     def get(self):
         """
-        Authenticate a Rucio account temporarily via an x509 certificate.
-
-        .. :quickref: x509; Authenticate with x509 certificate.
-
-        :reqheader Rucio-VO: VO name as a string (Multi-VO only).
-        :reqheader Rucio-Account: Account identifier as a string.
-        :reqheader Rucio-AppID: Application identifier as a string.
-        :reqheader SSLStdEnv: Apache mod_ssl SSL Standard Env Variables.
-        :resheader Access-Control-Allow-Origin:
-        :resheader Access-Control-Allow-Headers:
-        :resheader Access-Control-Allow-Methods:
-        :resheader Access-Control-Allow-Credentials:
-        :resheader Access-Control-Expose-Headers:
-        :resheader X-Rucio-Auth-Token: The authentication token
-        :status 200: Successfully authenticated
-        :status 404: Invalid credentials
+        ---
+        summary: x509
+        description: Authenticate a Rucio account via x509.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Rucio-Account
+          in: header
+          schema:
+            type: string
+          required: true
+        - name: X-Rucio-AppID
+          in: header
+          schema:
+            type: string
+        - name: X-Forwarded-For
+          in: header
+          schema:
+            type: string
+        responses:
+          200:
+            description: OK
+            headers:
+              X-Rucio-Auth-Token:
+                description: The authentication token
+                schema:
+                  type: string
+              X-Rucio-Auth-Token-Expires:
+                description: The time when the token expires
+                schema:
+                  type: string
+          401:
+            description: Cannot authenticate
         """
         headers = self.get_headers()
 
@@ -678,32 +1096,79 @@ class SSH(ErrorHandlingMethodView):
 
     def options(self):
         """
-        HTTP Success:
-            200 OK
-
-        Allow cross-site scripting. Explicit for Authentication.
+        ---
+        summary: SSH Allow cross-site scripting
+        description: SSH Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+              Access-Control-Expose-Headers:
+                schema:
+                  type: string
+                  enum: ['X-Rucio-Auth-Token']
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
     @check_accept_header_wrapper_flask(['application/octet-stream'])
     def get(self):
         """
-        Authenticate a Rucio account temporarily via SSH key exchange.
-
-        .. :quickref: SSH; Authenticate with SSH key exchange.
-
-        :reqheader Rucio-VO: VO name as a string (Multi-VO only).
-        :reqheader Rucio-Account: Account identifier as a string.
-        :reqheader Rucio-SSH-Signature: Response to server challenge signed with SSH private key as a base64 encoded string.
-        :reqheader Rucio-AppID: Application identifier as a string.
-        :resheader Access-Control-Allow-Origin:
-        :resheader Access-Control-Allow-Headers:
-        :resheader Access-Control-Allow-Methods:
-        :resheader Access-Control-Allow-Credentials:
-        :resheader Access-Control-Expose-Headers:
-        :resheader X-Rucio-Auth-Token: The authentication token
-        :status 200: Successfully authenticated
-        :status 404: Invalid credentials
+        ---
+        summary: SSH
+        description: Authenticate a Rucio account via SSH.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Rucio-Account
+          in: header
+          schema:
+            type: string
+          required: true
+        - name: X-Rucio-SSH-Signature
+          in: header
+          schema:
+            type: string
+          required: true
+        - name: X-Rucio-AppID
+          in: header
+          schema:
+            type: string
+        - name: X-Forwarded-For
+          in: header
+          schema:
+            type: string
+        responses:
+          200:
+            description: OK
+            headers:
+              X-Rucio-Auth-Token:
+                description: The authentication token
+                schema:
+                  type: string
+              X-Rucio-Auth-Token-Expires:
+                description: The time when the token expires
+                schema:
+                  type: string
+          401:
+            description: Cannot authenticate
         """
         headers = self.get_headers()
 
@@ -769,32 +1234,74 @@ class SSHChallengeToken(ErrorHandlingMethodView):
 
     def options(self):
         """
-        HTTP Success:
-            200 OK
-
-        Allow cross-site scripting. Explicit for Authentication.
+        ---
+        summary: SSHChallengeToken Allow cross-site scripting
+        description: SSHChallengeToken Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+              Access-Control-Expose-Headers:
+                schema:
+                  type: string
+                  enum: ['X-Rucio-Auth-Token']
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
     @check_accept_header_wrapper_flask(['application/octet-stream'])
     def get(self):
         """
-        Request a challenge token for SSH authentication
-
-        .. :quickref: SSH; Request SSH Challenge Token
-
-        :reqheader Rucio-VO: VO name as a string (Multi-VO only).
-        :reqheader Rucio-Account: Account identifier as a string.
-        :reqheader Rucio-AppID: Application identifier as a string.
-        :resheader Access-Control-Allow-Origin:
-        :resheader Access-Control-Allow-Headers:
-        :resheader Access-Control-Allow-Methods:
-        :resheader Access-Control-Allow-Credentials:
-        :resheader Access-Control-Expose-Headers:
-        :resheader X-Rucio-SSH-Challenge-Token: The SSH challenge token
-        :resheader X-Rucio-SSH-Challenge-Token-Expires: The expiry time of the token
-        :status 200: Successfully authenticated
-        :status 404: Invalid credentials
+        ---
+        summary: SSHChallengeToken
+        description: Authenticate a Rucio account via SSHChallengeToken.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Rucio-Account
+          in: header
+          schema:
+            type: string
+          required: true
+        - name: X-Rucio-AppID
+          in: header
+          schema:
+            type: string
+        - name: X-Forwarded-For
+          in: header
+          schema:
+            type: string
+        responses:
+          200:
+            description: OK
+            headers:
+              X-Rucio-SSH-Challenge-Token:
+                description: The authentication token
+                schema:
+                  type: string
+              X-Rucio-SSH-Challenge-Token-Expires:
+                description: The time when the token expires
+                schema:
+                  type: string
+          401:
+            description: Cannot authenticate
         """
         headers = self.get_headers()
 
@@ -839,25 +1346,78 @@ class SAML(ErrorHandlingMethodView):
 
     def options(self):
         """
-        Allow cross-site scripting. Explicit for Authentication.
-
-        :status 200: OK
+        ---
+        summary: SAML Allow cross-site scripting
+        description: SAML Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+              Access-Control-Expose-Headers:
+                schema:
+                  type: string
+                  enum: ['X-Rucio-Auth-Token']
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
     @check_accept_header_wrapper_flask(['application/octet-stream'])
     def get(self):
         """
-        .. :quickref: SAML;
-
-        :status 200: OK
-        :status 401: Unauthorized
-        :reqheader Rucio-VO: VO name as a string (Multi-VO only)
-        :reqheader Rucio-Account: Account identifier as a string.
-        :reqheader Rucio-Username: Username as a string.
-        :reqheader Rucio-Password: Password as a string.
-        :reqheader Rucio-AppID: Application identifier as a string.
-        :resheader X-Rucio-SAML-Auth-URL: as a variable-length string header.
+        ---
+        summary: SAML
+        description: Authenticate a Rucio account via SAML.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Rucio-Account
+          in: header
+          schema:
+            type: string
+          required: true
+        - name: X-Rucio-AppID
+          in: header
+          schema:
+            type: string
+        - name: X-Forwarded-For
+          in: header
+          schema:
+            type: string
+        responses:
+          200:
+            description: OK
+            headers:
+              X-Rucio-Auth-Token:
+                description: The authentication token
+                schema:
+                  type: string
+              X-Rucio-Auth-Token-Expires:
+                description: The time when the token expires
+                schema:
+                  type: string
+              X-Rucio-SAML-Auth-URL:
+                description: The time when the token expires
+                schema:
+                  type: string
+          401:
+            description: Cannot authenticate
         """
         headers = self.get_headers()
 
@@ -909,9 +1469,16 @@ class SAML(ErrorHandlingMethodView):
 
     def post(self):
         """
-        .. :quickref: SAML;
-
-        :status 200: OK
+        ---
+        summary: Post a SAML request
+        description: Post a SAML request
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+          401:
+            description: Invalid Auth Token
         """
         if not EXTRA_MODULES['onelogin']:
             return "SAML not configured on the server side.", 200, [('X-Rucio-Auth-Token', '')]
@@ -946,23 +1513,57 @@ class Validate(ErrorHandlingMethodView):
 
     def options(self):
         """
-        HTTP Success:
-            200 OK
-
-        Allow cross-site scripting. Explicit for Authentication.
+        ---
+        summary: Validate Allow cross-site scripting
+        description: Validate Allow cross-site scripting. Explicit for Authentication.
+        tags:
+          - Auth
+        responses:
+          200:
+            description: OK
+            headers:
+              Access-Control-Allow-Origin:
+                schema:
+                  type: string
+              Access-Control-Allow-Headers:
+                schema:
+                  type: string
+              Access-Control-Allow-Methods:
+                schema:
+                  type: string
+                  enum: ['*']
+              Access-Control-Allow-Credentials:
+                schema:
+                  type: string
+                  enum: ['true']
+              Access-Control-Expose-Headers:
+                schema:
+                  type: string
+                  enum: ['X-Rucio-Auth-Token']
+          404:
+            description: Not found
         """
         return '', 200, self.get_headers()
 
     @check_accept_header_wrapper_flask(['application/octet-stream'])
     def get(self):
         """
-        Validate a Rucio Auth Token.
-
-        .. :quickref: Validate; Validate a Rucio Auth Token.
-
-        :reqheader Rucio-Auth-Token: as a variable-length string.
-        :status 406: Not Acceptable.
-        :returns: Tuple(account name, token lifetime).
+        ---
+        summary: Validate
+        description: Validate a Rucio auth token.
+        tags:
+          - Auth
+        parameters:
+        - name: X-Rucio-Account
+          in: header
+          schema:
+            type: string
+          required: true
+        responses:
+          200:
+            description: OK
+          401:
+            description: Cannot authenticate
         """
 
         headers = self.get_headers()
